@@ -47,7 +47,13 @@ class NamespaceManager(object):
     implementation.
 
     """
+    
+    @classmethod
+    def _init_dependencies(cls):
+        pass
+        
     def __init__(self, namespace):
+        self._init_dependencies()
         self.namespace = namespace
         
     def get_creation_lock(self, key):
@@ -217,29 +223,26 @@ class Value(object):
         try:    
             has_value = self.namespace.has_key(self.key)
             if has_value:
-                value = self.__get_value()
-                return not self._is_expired()
-            else:
-                return False
+                try:
+                    stored, expired, value = self._get_value()
+                    return not self._is_expired(stored, expired)
+                except KeyError:
+                    pass
+            return False
         finally:
             self.namespace.release_read_lock()
 
-    def _is_expired(self):
-        """Return true if this container's value is expired.
-        
-        Note that this method is only correct if has_current_value()
-        or get_value() have been called already.
-        
-        """
+    def _is_expired(self, storedtime, expiretime):
+        """Return true if this container's value is expired."""
         return (
             (
                 self.starttime is not None and
-                self.storedtime < self.starttime
+                storedtime < self.starttime
             )
             or
             (
-                self.expiretime is not None and
-                time.time() >= self.expiretime + self.storedtime
+                expiretime is not None and
+                time.time() >= expiretime + storedtime
             )
         )
 
@@ -249,12 +252,12 @@ class Value(object):
             has_value = self.has_value()
             if has_value:
                 try:
-                    value = self.__get_value()
-                    if not self._is_expired():
+                    stored, expired, value = self._get_value()
+                    if not self._is_expired(stored, expired):
                         return value
                 except KeyError:
                     # guard against un-mutexed backends raising KeyError
-                    pass
+                    has_value = False
                     
             if not self.createfunc:
                 raise KeyError(self.key)
@@ -282,8 +285,8 @@ class Value(object):
             try:
                 if self.has_value():
                     try:
-                        value = self.__get_value()
-                        if not self._is_expired():
+                        stored, expired, value = self._get_value()
+                        if not self._is_expired(stored, expired):
                             return value
                     except KeyError:
                         # guard against un-mutexed backends raising KeyError
@@ -299,28 +302,34 @@ class Value(object):
             creation_lock.release()
             debug("released create lock")
 
-    def __get_value(self):
+    def _get_value(self):
         value = self.namespace[self.key]
         try:
-            self.storedtime, self.expiretime, value = value
+            stored, expired, value = value
         except ValueError:
             if not len(value) == 2:
                 raise
             # Old format: upgrade
-            self.storedtime, value = value
-            self.expiretime = self.expire_argument = None
-            debug("get_value upgrading time %r expire time %r", self.storedtime, self.expire_argument)
+            stored, value = value
+            expired = self.expire_argument
+            debug("get_value upgrading time %r expire time %r", stored, self.expire_argument)
             self.namespace.release_read_lock()
-            self.set_value(value)
+            self.set_value(value, stored)
             self.namespace.acquire_read_lock()
-        return value
+        except TypeError:
+            # occurs when the value is None.  memcached 
+            # may yank the rug from under us in which case 
+            # that's the result
+            raise KeyError(self.key)            
+        return stored, expired, value
 
-    def set_value(self, value):
+    def set_value(self, value, storedtime=None):
         self.namespace.acquire_write_lock()
         try:
-            self.storedtime = time.time()
-            debug("set_value stored time %r expire time %r", self.storedtime, self.expire_argument)
-            self.namespace.set_value(self.key, (self.storedtime, self.expire_argument, value))
+            if storedtime is None:
+                storedtime = time.time()
+            debug("set_value stored time %r expire time %r", storedtime, self.expire_argument)
+            self.namespace.set_value(self.key, (storedtime, self.expire_argument, value))
         finally:
             self.namespace.release_write_lock()
 
@@ -338,17 +347,34 @@ class Value(object):
         finally:
             self.namespace.release_write_lock()
 
+class AbstractDictionaryNSManager(NamespaceManager):
+    """A subclassable NamespaceManager that places data in a dictionary.
+    
+    Subclasses should provide a "dictionary" attribute or descriptor
+    which returns a dict-like object.   The dictionary will store keys
+    that are local to the "namespace" attribute of this manager, so
+    ensure that the dictionary will not be used by any other namespace.
 
-class MemoryNamespaceManager(NamespaceManager):
-    namespaces = util.SyncDict()
-
-    def __init__(self, namespace, **kwargs):
-        NamespaceManager.__init__(self, namespace)
-        self.dictionary = MemoryNamespaceManager.namespaces.get(self.namespace,
-                                                                dict)
+    e.g.::
+    
+        import collections
+        cached_data = collections.defaultdict(dict)
+        
+        class MyDictionaryManager(AbstractDictionaryNSManager):
+            def __init__(self, namespace):
+                AbstractDictionaryNSManager.__init__(self, namespace)
+                self.dictionary = cached_data[self.namespace]
+                
+    The above stores data in a global dictionary called "cached_data",
+    which is structured as a dictionary of dictionaries, keyed
+    first on namespace name to a sub-dictionary, then on actual
+    cache key to value.
+    
+    """
+    
     def get_creation_lock(self, key):
         return NameLock(
-            identifier="memorycontainer/funclock/%s/%s" % (self.namespace, key),
+            identifier="memorynamespace/funclock/%s/%s" % (self.namespace, key),
             reentrant=True
         )
 
@@ -372,7 +398,14 @@ class MemoryNamespaceManager(NamespaceManager):
         
     def keys(self):
         return self.dictionary.keys()
+    
+class MemoryNamespaceManager(AbstractDictionaryNSManager):
+    namespaces = util.SyncDict()
 
+    def __init__(self, namespace, **kwargs):
+        AbstractDictionaryNSManager.__init__(self, namespace)
+        self.dictionary = MemoryNamespaceManager.namespaces.get(self.namespace,
+                                                                dict)
 
 class DBMNamespaceManager(OpenResourceNamespaceManager):
     def __init__(self, namespace, dbmmodule=None, data_dir=None, 
@@ -540,7 +573,12 @@ class FileNamespaceManager(OpenResourceNamespaceManager):
         self.flags = None
                 
     def do_remove(self):
-        os.remove(self.file)
+        try:
+            os.remove(self.file)
+        except OSError, err:
+            # for instance, because we haven't yet used this cache,
+            # but client code has asked for a clear() operation...
+            pass
         self.hash = {}
         
     def __getitem__(self, key): 
