@@ -5,11 +5,10 @@ class BeakerException(Exception):
 
 
 class CreationAbortedError(Exception):
-    """an exception that allows a creation function to abort what it's
-    doing"""
+    """Deprecated."""
 
 
-class InvalidCacheBackendError(BeakerException):
+class InvalidCacheBackendError(BeakerException, ImportError):
     pass
 
 
@@ -19,3 +18,7 @@ class MissingCacheParameter(BeakerException):
 
 class LockError(BeakerException):
     pass
+
+
+class InvalidCryptoBackendError(BeakerException):
+    pass
